@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 1997-9, 2000, 2006, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000, 2006, 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -76,6 +76,12 @@ struct variable
     struct attrset attributes;
   };
 
+
+static void var_set_print_format_quiet (struct variable *v, const struct fmt_spec *print);
+static void var_set_write_format_quiet (struct variable *v, const struct fmt_spec *write);
+static bool var_set_label_quiet (struct variable *v, const char *label, bool issue_warning);
+static void var_set_name_quiet (struct variable *v, const char *name);
+
 /* Creates and returns a new variable with the given NAME and
    WIDTH and other fields initialized to default values.  The
    variable is not added to a dictionary; for that, use
@@ -89,7 +95,7 @@ var_create (const char *name, int width)
   assert (width >= 0 && width <= MAX_STRING);
 
   v = xzalloc (sizeof *v);
-  var_set_name (v, name);
+  var_set_name_quiet (v, name);
   v->width = width;
   mv_init (&v->miss, width);
   v->leave = var_must_leave (v);
@@ -104,36 +110,6 @@ var_create (const char *name, int width)
   return v;
 }
 
-/* Creates and returns a clone of OLD_VAR.  Most properties of
-   the new variable are copied from OLD_VAR, except:
-
-    - The variable's short name is not copied, because there is
-      no reason to give a new variable with potentially a new
-      name the same short name.
-
-    - The new variable is not added to OLD_VAR's dictionary by
-      default.  Use dict_clone_var, instead, to do that.
-*/
-struct variable *
-var_clone (const struct variable *old_var)
-{
-  struct variable *new_var = var_create (var_get_name (old_var),
-                                         var_get_width (old_var));
-
-  var_set_missing_values (new_var, var_get_missing_values (old_var));
-  var_set_print_format (new_var, var_get_print_format (old_var));
-  var_set_write_format (new_var, var_get_write_format (old_var));
-  var_set_value_labels (new_var, var_get_value_labels (old_var));
-  var_set_label (new_var, var_get_label (old_var), false);
-  var_set_measure (new_var, var_get_measure (old_var));
-  var_set_display_width (new_var, var_get_display_width (old_var));
-  var_set_alignment (new_var, var_get_alignment (old_var));
-  var_set_leave (new_var, var_get_leave (old_var));
-  var_set_attributes (new_var, var_get_attributes (old_var));
-
-  return new_var;
-}
-
 /* Destroys variable V.
    V must not belong to a dictionary.  If it does, use
    dict_delete_var instead. */
@@ -146,7 +122,7 @@ var_destroy (struct variable *v)
       mv_destroy (&v->miss);
       var_clear_short_names (v);
       val_labs_destroy (v->val_labs);
-      var_clear_label (v);
+      var_set_label_quiet (v, NULL, false);
       attrset_destroy (var_get_attributes (v));
       free (v->name);
       ds_destroy (&v->name_and_label);
@@ -168,8 +144,8 @@ var_get_name (const struct variable *v)
 /* Sets V's name to NAME, a UTF-8 encoded string.
    Do not use this function for a variable in a dictionary.  Use
    dict_rename_var instead. */
-void
-var_set_name (struct variable *v, const char *name)
+static void
+var_set_name_quiet (struct variable *v, const char *name)
 {
   assert (!var_has_vardict (v));
   assert (id_is_plausible (name, false));
@@ -178,7 +154,17 @@ var_set_name (struct variable *v, const char *name)
   v->name = xstrdup (name);
   ds_destroy (&v->name_and_label);
   ds_init_empty (&v->name_and_label);
-  dict_var_changed (v);
+}
+
+/* Sets V's name to NAME, a UTF-8 encoded string.
+   Do not use this function for a variable in a dictionary.  Use
+   dict_rename_var instead. */
+void
+var_set_name (struct variable *v, const char *name)
+{
+  struct variable *ov = var_clone (v);
+  var_set_name_quiet (v, name);
+  dict_var_changed (v, VAR_TRAIT_NAME, ov);
 }
 
 /* Returns VAR's dictionary class. */
@@ -258,22 +244,25 @@ var_get_width (const struct variable *v)
   return v->width;
 }
 
-/* Changes the width of V to NEW_WIDTH.
-   This function should be used cautiously. */
 void
-var_set_width (struct variable *v, int new_width)
+var_set_width_and_formats (struct variable *v, int new_width,
+			   const struct fmt_spec *print, const struct fmt_spec *write)
 {
-  const int old_width = v->width;
+  struct variable *ov;
+  unsigned int traits = 0;
 
-  if (old_width == new_width)
-    return;
+  ov = var_clone (v);
 
-  if (mv_is_resizable (&v->miss, new_width))
-    mv_resize (&v->miss, new_width);
-  else
+  if (var_has_missing_values (v))
     {
-      mv_destroy (&v->miss);
-      mv_init (&v->miss, new_width);
+      if (mv_is_resizable (&v->miss, new_width))
+	mv_resize (&v->miss, new_width);
+      else
+	{
+	  mv_destroy (&v->miss);
+	  mv_init (&v->miss, new_width);
+	}
+      traits |= VAR_TRAIT_MISSING_VALUES;
     }
 
   if (v->val_labs != NULL)
@@ -285,16 +274,49 @@ var_set_width (struct variable *v, int new_width)
           val_labs_destroy (v->val_labs);
           v->val_labs = NULL;
         }
+      traits |= VAR_TRAIT_VALUE_LABELS;
     }
 
-  fmt_resize (&v->print, new_width);
-  fmt_resize (&v->write, new_width);
+  if (fmt_resize (&v->print, new_width))
+    traits |= VAR_TRAIT_PRINT_FORMAT;
+
+  if (fmt_resize (&v->write, new_width))
+    traits |= VAR_TRAIT_WRITE_FORMAT;
 
   v->width = new_width;
-  dict_var_resized (v, old_width);
-  dict_var_changed (v);
+  traits |= VAR_TRAIT_WIDTH;
+
+  if (print)
+    {
+      var_set_print_format_quiet (v, print);
+      traits |= VAR_TRAIT_PRINT_FORMAT;
+    }
+
+  if (write)
+    {
+      var_set_write_format_quiet (v, write);
+      traits |= VAR_TRAIT_WRITE_FORMAT;
+    }
+
+  dict_var_changed (v, traits, ov);
 }
 
+/* Changes the width of V to NEW_WIDTH.
+   This function should be used cautiously. */
+void
+var_set_width (struct variable *v, int new_width)
+{
+  const int old_width = v->width;
+
+  if (old_width == new_width)
+    return;
+
+  var_set_width_and_formats (v, new_width, NULL, NULL);
+}
+
+
+
+
 /* Returns true if variable V is numeric, false otherwise. */
 bool
 var_is_numeric (const struct variable *v)
@@ -321,8 +343,8 @@ var_get_missing_values (const struct variable *v)
    width or at least resizable to V's width.
    If MISS is null, then V's missing values, if any, are
    cleared. */
-void
-var_set_missing_values (struct variable *v, const struct missing_values *miss)
+static void
+var_set_missing_values_quiet (struct variable *v, const struct missing_values *miss)
 {
   if (miss != NULL)
     {
@@ -333,8 +355,18 @@ var_set_missing_values (struct variable *v, const struct missing_values *miss)
     }
   else
     mv_clear (&v->miss);
+}
 
-  dict_var_changed (v);
+/* Sets variable V's missing values to MISS, which must be of V's
+   width or at least resizable to V's width.
+   If MISS is null, then V's missing values, if any, are
+   cleared. */
+void
+var_set_missing_values (struct variable *v, const struct missing_values *miss)
+{
+  struct variable *ov = var_clone (v);
+  var_set_missing_values_quiet (v, miss);
+  dict_var_changed (v, VAR_TRAIT_MISSING_VALUES, ov);
 }
 
 /* Sets variable V to have no user-missing values. */
@@ -399,8 +431,8 @@ var_has_value_labels (const struct variable *v)
    which must have a width equal to V's width or one that can be
    changed to V's width.
    If VLS is null, then V's value labels, if any, are removed. */
-void
-var_set_value_labels (struct variable *v, const struct val_labs *vls)
+static void
+var_set_value_labels_quiet (struct variable *v, const struct val_labs *vls)
 {
   val_labs_destroy (v->val_labs);
   v->val_labs = NULL;
@@ -410,10 +442,23 @@ var_set_value_labels (struct variable *v, const struct val_labs *vls)
       assert (val_labs_can_set_width (vls, v->width));
       v->val_labs = val_labs_clone (vls);
       val_labs_set_width (v->val_labs, v->width);
-      dict_var_changed (v);
     }
 }
 
+
+/* Sets variable V's value labels to a copy of VLS,
+   which must have a width equal to V's width or one that can be
+   changed to V's width.
+   If VLS is null, then V's value labels, if any, are removed. */
+void
+var_set_value_labels (struct variable *v, const struct val_labs *vls)
+{
+  struct variable *ov = var_clone (v);
+  var_set_value_labels_quiet (v, vls);
+  dict_var_changed (v, VAR_TRAIT_LABEL, ov);  
+}
+
+
 /* Makes sure that V has a set of value labels,
    by assigning one to it if necessary. */
 static void
@@ -527,17 +572,28 @@ var_get_print_format (const struct variable *v)
    valid format specification for a variable of V's width
    (ordinarily an output format, but input formats are not
    rejected). */
-void
-var_set_print_format (struct variable *v, const struct fmt_spec *print)
+static void
+var_set_print_format_quiet (struct variable *v, const struct fmt_spec *print)
 {
   if (!fmt_equal (&v->print, print))
     {
       assert (fmt_check_width_compat (print, v->width));
       v->print = *print;
-      dict_var_changed (v);
     }
 }
 
+/* Sets V's print format specification to PRINT, which must be a
+   valid format specification for a variable of V's width
+   (ordinarily an output format, but input formats are not
+   rejected). */
+void
+var_set_print_format (struct variable *v, const struct fmt_spec *print)
+{
+  struct variable *ov = var_clone (v);
+  var_set_print_format_quiet (v, print);
+  dict_var_changed (v, VAR_TRAIT_PRINT_FORMAT, ov);
+}
+
 /* Returns V's write format specification. */
 const struct fmt_spec *
 var_get_write_format (const struct variable *v)
@@ -549,17 +605,29 @@ var_get_write_format (const struct variable *v)
    valid format specification for a variable of V's width
    (ordinarily an output format, but input formats are not
    rejected). */
-void
-var_set_write_format (struct variable *v, const struct fmt_spec *write)
+static void
+var_set_write_format_quiet (struct variable *v, const struct fmt_spec *write)
 {
   if (!fmt_equal (&v->write, write))
     {
       assert (fmt_check_width_compat (write, v->width));
       v->write = *write;
-      dict_var_changed (v);
     }
 }
 
+/* Sets V's write format specification to WRITE, which must be a
+   valid format specification for a variable of V's width
+   (ordinarily an output format, but input formats are not
+   rejected). */
+void
+var_set_write_format (struct variable *v, const struct fmt_spec *write)
+{
+  struct variable *ov = var_clone (v);
+  var_set_write_format_quiet (v, write);
+  dict_var_changed (v, VAR_TRAIT_WRITE_FORMAT, ov);
+}
+
+
 /* Sets V's print and write format specifications to FORMAT,
    which must be a valid format specification for a variable of
    V's width (ordinarily an output format, but input formats are
@@ -567,8 +635,10 @@ var_set_write_format (struct variable *v, const struct fmt_spec *write)
 void
 var_set_both_formats (struct variable *v, const struct fmt_spec *format)
 {
-  var_set_print_format (v, format);
-  var_set_write_format (v, format);
+  struct variable *ov = var_clone (v);
+  var_set_print_format_quiet (v, format);
+  var_set_write_format_quiet (v, format);
+  dict_var_changed (v, VAR_TRAIT_PRINT_FORMAT | VAR_TRAIT_WRITE_FORMAT, ov);
 }
 
 /* Returns the default print and write format for a variable of
@@ -645,8 +715,8 @@ var_get_label (const struct variable *v)
    var_get_encoding()).  If LABEL fits within this limit, this function returns
    true.  Otherwise, the variable label is set to a truncated value, this
    function returns false and, if ISSUE_WARNING is true, issues a warning.  */
-bool
-var_set_label (struct variable *v, const char *label, bool issue_warning)
+static bool
+var_set_label_quiet (struct variable *v, const char *label, bool issue_warning)
 {
   bool truncated = false;
 
@@ -681,11 +751,32 @@ var_set_label (struct variable *v, const char *label, bool issue_warning)
   ds_destroy (&v->name_and_label);
   ds_init_empty (&v->name_and_label);
 
-  dict_var_changed (v);
+  return truncated;
+}
+
+
+
+/* Sets V's variable label to UTF-8 encoded string LABEL, stripping off leading
+   and trailing white space.  If LABEL is a null pointer or if LABEL is an
+   empty string (after stripping white space), then V's variable label (if any)
+   is removed.
+
+   Variable labels are limited to 255 bytes in V's encoding (as returned by
+   var_get_encoding()).  If LABEL fits within this limit, this function returns
+   true.  Otherwise, the variable label is set to a truncated value, this
+   function returns false and, if ISSUE_WARNING is true, issues a warning.  */
+bool
+var_set_label (struct variable *v, const char *label, bool issue_warning)
+{
+  struct variable *ov = var_clone (v);
+  bool truncated = var_set_label_quiet (v, label, issue_warning);
+
+  dict_var_changed (v, VAR_TRAIT_LABEL, ov);
 
   return truncated;
 }
 
+
 /* Removes any variable label from V. */
 void
 var_clear_label (struct variable *v)
@@ -737,14 +828,24 @@ var_get_measure (const struct variable *v)
 }
 
 /* Sets V's measurement level to MEASURE. */
-void
-var_set_measure (struct variable *v, enum measure measure)
+static void
+var_set_measure_quiet (struct variable *v, enum measure measure)
 {
   assert (measure_is_valid (measure));
   v->measure = measure;
-  dict_var_changed (v);
 }
 
+
+/* Sets V's measurement level to MEASURE. */
+void
+var_set_measure (struct variable *v, enum measure measure)
+{
+  struct variable *ov = var_clone (v);
+  var_set_measure_quiet (v, measure);
+  dict_var_changed (v, VAR_TRAIT_MEASURE, ov);
+}
+
+
 /* Returns the default measurement level for a variable of the
    given TYPE, as set by var_create.  The return value can be
    used to reset a variable's measurement level to the
@@ -763,17 +864,24 @@ var_get_display_width (const struct variable *v)
 }
 
 /* Sets V's display width to DISPLAY_WIDTH. */
-void
-var_set_display_width (struct variable *v, int new_width)
+static void
+var_set_display_width_quiet (struct variable *v, int new_width)
 {
   if (v->display_width != new_width)
     {
       v->display_width = new_width;
-      dict_var_display_width_changed (v);
-      dict_var_changed (v);
     }
 }
 
+void
+var_set_display_width (struct variable *v, int new_width)
+{
+  struct variable *ov = var_clone (v);
+  var_set_display_width_quiet (v, new_width);
+  dict_var_changed (v, VAR_TRAIT_DISPLAY_WIDTH, ov);
+}
+
+
 /* Returns the default display width for a variable of the given
    WIDTH, as set by var_create.  The return value can be used to
    reset a variable's display width to the default. */
@@ -819,14 +927,23 @@ var_get_alignment (const struct variable *v)
 }
 
 /* Sets V's display alignment to ALIGNMENT. */
-void
-var_set_alignment (struct variable *v, enum alignment alignment)
+static void
+var_set_alignment_quiet (struct variable *v, enum alignment alignment)
 {
   assert (alignment_is_valid (alignment));
   v->alignment = alignment;
-  dict_var_changed (v);
 }
 
+/* Sets V's display alignment to ALIGNMENT. */
+void
+var_set_alignment (struct variable *v, enum alignment alignment)
+{
+  struct variable *ov = var_clone (v);
+  var_set_alignment_quiet (v, alignment);
+  dict_var_changed (v, VAR_TRAIT_ALIGNMENT, ov);
+}
+
+
 /* Returns the default display alignment for a variable of the
    given TYPE, as set by var_create.  The return value can be
    used to reset a variable's display alignment to the default. */
@@ -848,14 +965,24 @@ var_get_leave (const struct variable *v)
 }
 
 /* Sets V's leave setting to LEAVE. */
-void
-var_set_leave (struct variable *v, bool leave)
+static void
+var_set_leave_quiet (struct variable *v, bool leave)
 {
   assert (leave || !var_must_leave (v));
   v->leave = leave;
-  dict_var_changed (v);
 }
 
+
+/* Sets V's leave setting to LEAVE. */
+void
+var_set_leave (struct variable *v, bool leave)
+{
+  struct variable *ov = var_clone (v);
+  var_set_leave_quiet (v, leave);
+  dict_var_changed (v, VAR_TRAIT_LEAVE, ov);
+}
+
+
 /* Returns true if V must be left from case to case,
    false if it can be set either way. */
 bool
@@ -899,6 +1026,8 @@ var_get_short_name (const struct variable *var, size_t idx)
 void
 var_set_short_name (struct variable *var, size_t idx, const char *short_name)
 {
+  struct variable *ov = var_clone (var);
+
   assert (short_name == NULL || id_is_plausible (short_name, false));
 
   /* Clear old short name numbered IDX, if any. */
@@ -924,7 +1053,7 @@ var_set_short_name (struct variable *var, size_t idx, const char *short_name)
       var->short_names[idx] = utf8_to_upper (short_name);
     }
 
-  dict_var_changed (var);
+  dict_var_changed (var, VAR_TRAIT_NAME, ov);
 }
 
 /* Clears V's short names. */
@@ -974,13 +1103,23 @@ var_get_attributes (const struct variable *v)
 }
 
 /* Replaces variable V's attributes set by a copy of ATTRS. */
-void
-var_set_attributes (struct variable *v, const struct attrset *attrs) 
+static void
+var_set_attributes_quiet (struct variable *v, const struct attrset *attrs) 
 {
   attrset_destroy (&v->attributes);
   attrset_clone (&v->attributes, attrs);
 }
 
+/* Replaces variable V's attributes set by a copy of ATTRS. */
+void
+var_set_attributes (struct variable *v, const struct attrset *attrs) 
+{
+  struct variable *ov = var_clone (v);
+  var_set_attributes_quiet (v, attrs);
+  dict_var_changed (v, VAR_TRAIT_ATTRIBUTES, ov);
+}
+
+
 /* Returns true if V has any custom attributes, false if it has none. */
 bool
 var_has_attributes (const struct variable *v)
@@ -988,6 +1127,39 @@ var_has_attributes (const struct variable *v)
   return attrset_count (&v->attributes) > 0;
 }
 
+
+/* Creates and returns a clone of OLD_VAR.  Most properties of
+   the new variable are copied from OLD_VAR, except:
+
+    - The variable's short name is not copied, because there is
+      no reason to give a new variable with potentially a new
+      name the same short name.
+
+    - The new variable is not added to OLD_VAR's dictionary by
+      default.  Use dict_clone_var, instead, to do that.
+*/
+struct variable *
+var_clone (const struct variable *old_var)
+{
+  struct variable *new_var = var_create (var_get_name (old_var),
+                                         var_get_width (old_var));
+
+  var_set_missing_values_quiet (new_var, var_get_missing_values (old_var));
+  var_set_print_format_quiet (new_var, var_get_print_format (old_var));
+  var_set_write_format_quiet (new_var, var_get_write_format (old_var));
+  var_set_value_labels_quiet (new_var, var_get_value_labels (old_var));
+  var_set_label_quiet (new_var, var_get_label (old_var), false);
+  var_set_measure_quiet (new_var, var_get_measure (old_var));
+  var_set_display_width_quiet (new_var, var_get_display_width (old_var));
+  var_set_alignment_quiet (new_var, var_get_alignment (old_var));
+  var_set_leave_quiet (new_var, var_get_leave (old_var));
+  var_set_attributes_quiet (new_var, var_get_attributes (old_var));
+
+  return new_var;
+}
+
+
+
 /* Returns the encoding of values of variable VAR.  (This is actually a
    property of the dictionary.)  Returns null if no specific encoding has been
    set.  */
