@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 2010 Free Software Foundation, Inc.
+   Copyright (C) 2010, 2011, 2013 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
   enum { CSV_FILE = 1, TAB_FILE } type;
 
   struct dictionary *dict;
+  struct case_map_stage *stage;
   struct case_map *map;
   struct casewriter *writer;
   struct file_handle *handle;
@@ -67,6 +68,7 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
   type = 0;
 
   dict = dict_clone (dataset_dict (ds));
+  stage = NULL;
   map = NULL;
 
   handle = NULL;
@@ -81,7 +83,7 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
   delimiter = 0;
   qualifier = '"';
 
-  case_map_prepare_dict (dict);
+  stage = case_map_stage_create (dict);
   dict_delete_scratch_vars (dict);
 
   while (lex_match (lexer, T_SLASH))
@@ -115,7 +117,7 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
             type = TAB_FILE;
           else
             {
-              lex_error (lexer, _("expecting %s or %s"), "CSV", "TAB");
+              lex_error_expecting (lexer, "CSV", "TAB", NULL_SENTINEL);
               goto error;
             }
         }
@@ -132,7 +134,7 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
             recode_user_missing = true;
           else
             {
-              lex_error (lexer, _("expecting %s or %s"), "IGNORE", "RECODE");
+              lex_error_expecting (lexer, "IGNORE", "RECODE", NULL_SENTINEL);
               goto error;
             }
         }
@@ -145,7 +147,7 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
             use_value_labels = true;
           else
             {
-              lex_error (lexer, _("expecting %s or %s"), "VALUES", "LABELS");
+              lex_error_expecting (lexer, "VALUES", "LABELS", NULL_SENTINEL);
               goto error;
             }
         }
@@ -193,8 +195,8 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
                     decimal = ',';
                   else
                     {
-                      lex_error (lexer, _("expecting %s or %s"),
-                                 "DOT", "COMMA");
+                      lex_error_expecting (lexer, "DOT", "COMMA",
+                                           NULL_SENTINEL);
                       goto error;
                     }
                 }
@@ -207,8 +209,8 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
                     use_print_formats = true;
                   else
                     {
-                      lex_error (lexer, _("expecting %s or %s"),
-                                 "PLAIN", "VARIABLE");
+                      lex_error_expecting (lexer, "PLAIN", "VARIABLE",
+                                           NULL_SENTINEL);
                       goto error;
                     }
                 }
@@ -225,7 +227,7 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
             retain_unselected = false;
           else
             {
-              lex_error (lexer, _("expecting %s or %s"), "RETAIN", "DELETE");
+              lex_error_expecting (lexer, "RETAIN", "DELETE", NULL_SENTINEL);
               goto error;
             }
         }
@@ -237,12 +239,12 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
 
   if (type == 0)
     {
-      lex_sbc_missing (lexer, "TYPE");
+      lex_sbc_missing ("TYPE");
       goto error;
     }
   else if (handle == NULL)
     {
-      lex_sbc_missing (lexer, "OUTFILE");
+      lex_sbc_missing ("OUTFILE");
       goto error;
     }
   else if (!replace && fn_exists (fh_get_file_name (handle)))
@@ -271,7 +273,8 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
     goto error;
   fh_unref (handle);
 
-  map = case_map_from_dict (dict);
+  map = case_map_stage_get_case_map (stage);
+  case_map_stage_destroy (stage);
   if (map != NULL)
     writer = case_map_create_output_translator (map, writer);
   dict_destroy (dict);
@@ -283,6 +286,7 @@ cmd_save_translate (struct lexer *lexer, struct dataset *ds)
   return ok ? CMD_SUCCESS : CMD_CASCADING_FAILURE;
 
 error:
+  case_map_stage_destroy (stage);
   fh_unref (handle);
   dict_destroy (dict);
   case_map_destroy (map);
