@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 1997-9, 2000, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000, 2010, 2011, 2012 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "libpspp/assertion.h"
 #include "libpspp/bit-vector.h"
 #include "libpspp/compiler.h"
+#include "libpspp/i18n.h"
 #include "libpspp/message.h"
 #include "libpspp/misc.h"
 #include "libpspp/str.h"
@@ -110,7 +111,7 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 
 	  if (already_encountered & 1)
 	    {
-	      msg (SE, _("%s subcommand may be given at most once."), "REORDER");
+              lex_sbc_only_once ("REORDER");
 	      goto done;
 	    }
 	  already_encountered |= 1;
@@ -143,7 +144,7 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 		{
 		  if (!lex_match (lexer, T_LPAREN))
 		    {
-		      msg (SE, _("`(' expected on %s subcommand."), "REORDER");
+                      lex_error_expecting (lexer, "`('", NULL_SENTINEL);
 		      free (v);
 		      goto done;
 		    }
@@ -155,8 +156,7 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 		    }
 		  if (!lex_match (lexer, T_RPAREN))
 		    {
-		      msg (SE, _("`)' expected following variable names on "
-			   "REORDER subcommand."));
+                      lex_error_expecting (lexer, "`)'", NULL_SENTINEL);
 		      free (v);
 		      goto done;
 		    }
@@ -174,7 +174,7 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 	{
 	  if (already_encountered & 2)
 	    {
-	      msg (SE, _("%s subcommand may be given at most once."), "RENAME");
+              lex_sbc_only_once ("RENAME");
 	      goto done;
 	    }
 	  already_encountered |= 2;
@@ -187,7 +187,7 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 
 	      if (!lex_match (lexer, T_LPAREN))
 		{
-		  msg (SE, _("`(' expected on %s subcommand."), "RENAME");
+                  lex_error_expecting (lexer, "`('", NULL_SENTINEL);
 		  goto done;
 		}
 	      if (!parse_variables (lexer, dataset_dict (ds),
@@ -196,8 +196,7 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 		goto done;
 	      if (!lex_match (lexer, T_EQUALS))
 		{
-		  msg (SE, _("`=' expected between lists of new and old variable "
-		       "names on RENAME subcommand."));
+                  lex_error_expecting (lexer, "`='", NULL_SENTINEL);
 		  goto done;
 		}
 	      if (!parse_DATA_LIST_vars (lexer, dataset_dict (ds),
@@ -216,8 +215,7 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 		}
 	      if (!lex_match (lexer, T_RPAREN))
 		{
-		  msg (SE, _("`)' expected after variable lists on RENAME "
-		       "subcommand."));
+                  lex_error_expecting (lexer, "`)'", NULL_SENTINEL);
 		  goto done;
 		}
 	    }
@@ -310,7 +308,7 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 	break;
       if (lex_token (lexer) != T_SLASH)
 	{
-	  msg (SE, _("`/' or `.' expected."));
+          lex_error_expecting (lexer, "`/'", "`.'", NULL_SENTINEL);
 	  goto done;
 	}
       lex_get (lexer);
@@ -358,7 +356,7 @@ compare_variables_given_ordering (const void *a_, const void *b_,
       result = a_index < b_index ? -1 : a_index > b_index;
     }
   else
-    result = strcasecmp (var_get_name (a), var_get_name (b));
+    result = utf8_strcasecmp (var_get_name (a), var_get_name (b));
   if (!ordering->forward)
     result = -result;
   return result;
@@ -380,7 +378,7 @@ compare_var_renaming_by_new_name (const void *a_, const void *b_,
   const struct var_renaming *a = a_;
   const struct var_renaming *b = b_;
 
-  return strcasecmp (a->new_name, b->new_name);
+  return utf8_strcasecmp (a->new_name, b->new_name);
 }
 
 /* Returns true if performing VM on dictionary D would not cause
