@@ -53,19 +53,13 @@ cmd_rename_variables (struct lexer *lexer, struct dataset *ds)
       size_t prev_nv_1 = rename_cnt;
       size_t prev_nv_2 = rename_cnt;
 
-      if (!lex_match (lexer, T_LPAREN))
-	{
-	  msg (SE, _("`(' expected."));
-	  goto lossage;
-	}
+      if (!lex_force_match (lexer, T_LPAREN))
+        goto lossage;
       if (!parse_variables (lexer, dataset_dict (ds), &rename_vars, &rename_cnt,
 			    PV_APPEND | PV_NO_DUPLICATE))
 	goto lossage;
-      if (!lex_match (lexer, T_EQUALS))
-	{
-	  msg (SE, _("`=' expected between lists of new and old variable names."));
-	  goto lossage;
-	}
+      if (!lex_force_match (lexer, T_EQUALS))
+        goto lossage;
       if (!parse_DATA_LIST_vars (lexer, dataset_dict (ds),
                                  &rename_new_names, &prev_nv_1,
                                  PV_APPEND | PV_NO_DUPLICATE))
@@ -83,11 +77,8 @@ cmd_rename_variables (struct lexer *lexer, struct dataset *ds)
 	  rename_new_names = NULL;
 	  goto lossage;
 	}
-      if (!lex_match (lexer, T_RPAREN))
-	{
-	  msg (SE, _("`)' expected after variable names."));
-	  goto lossage;
-	}
+      if (!lex_force_match (lexer, T_RPAREN))
+        goto lossage;
     }
   while (lex_token (lexer) != T_ENDCMD);
 
