@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000, 2010, 2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,16 +19,16 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <data/procedure.h>
-#include <data/variable.h>
-#include <language/command.h>
-#include <language/lexer/lexer.h>
-#include <language/lexer/variable-parser.h>
-#include <libpspp/message.h>
-#include <libpspp/str.h>
+#include "data/dataset.h"
+#include "data/variable.h"
+#include "language/command.h"
+#include "language/lexer/lexer.h"
+#include "language/lexer/variable-parser.h"
+#include "libpspp/message.h"
+#include "libpspp/str.h"
 
-#include "minmax.h"
-#include "xalloc.h"
+#include "gl/minmax.h"
+#include "gl/xalloc.h"
 
 #include "gettext.h"
 #define _(msgid) gettext (msgid)
@@ -51,7 +51,7 @@ cmd_variable_alignment (struct lexer *lexer, struct dataset *ds)
       if (!parse_variables (lexer, dataset_dict (ds), &v, &nv, PV_NONE))
         return CMD_FAILURE;
 
-      if ( lex_force_match (lexer, '(') )
+      if ( lex_force_match (lexer, T_LPAREN) )
 	{
 	  if ( lex_match_id (lexer, "LEFT"))
 	    align = ALIGN_LEFT;
@@ -65,7 +65,7 @@ cmd_variable_alignment (struct lexer *lexer, struct dataset *ds)
               return CMD_FAILURE;
             }
 
-	  lex_force_match (lexer, ')');
+	  lex_force_match (lexer, T_RPAREN);
 	}
       else
         {
@@ -76,12 +76,12 @@ cmd_variable_alignment (struct lexer *lexer, struct dataset *ds)
       for( i = 0 ; i < nv ; ++i )
         var_set_alignment (v[i], align);
 
-      while (lex_token (lexer) == '/')
+      while (lex_token (lexer) == T_SLASH)
 	lex_get (lexer);
       free (v);
 
     }
-  while (lex_token (lexer) != '.');
+  while (lex_token (lexer) != T_ENDCMD);
   return CMD_SUCCESS;
 }
 
@@ -102,14 +102,14 @@ cmd_variable_width (struct lexer *lexer, struct dataset *ds)
       if (!parse_variables (lexer, dataset_dict (ds), &v, &nv, PV_NONE))
         return CMD_FAILURE;
 
-      if (!lex_force_match (lexer, '(') || !lex_force_int (lexer))
+      if (!lex_force_match (lexer, T_LPAREN) || !lex_force_int (lexer))
         {
           free (v);
           return CMD_FAILURE;
         }
       width = lex_integer (lexer);
       lex_get (lexer);
-      if (!lex_force_match (lexer, ')'))
+      if (!lex_force_match (lexer, T_RPAREN))
         {
           free (v);
           return CMD_FAILURE;
@@ -126,12 +126,12 @@ cmd_variable_width (struct lexer *lexer, struct dataset *ds)
       for( i = 0 ; i < nv ; ++i )
         var_set_display_width (v[i], width);
 
-      while (lex_token (lexer) == '/')
+      while (lex_token (lexer) == T_SLASH)
 	lex_get (lexer);
       free (v);
 
     }
-  while (lex_token (lexer) != '.');
+  while (lex_token (lexer) != T_ENDCMD);
   return CMD_SUCCESS;
 }
 
@@ -149,7 +149,7 @@ cmd_variable_level (struct lexer *lexer, struct dataset *ds)
       if (!parse_variables (lexer, dataset_dict (ds), &v, &nv, PV_NONE))
         return CMD_FAILURE;
 
-      if ( lex_force_match (lexer, '(') )
+      if ( lex_force_match (lexer, T_LPAREN) )
 	{
 	  if ( lex_match_id (lexer, "SCALE"))
 	    level = MEASURE_SCALE;
@@ -163,7 +163,7 @@ cmd_variable_level (struct lexer *lexer, struct dataset *ds)
               return CMD_FAILURE;
             }
 
-	  lex_force_match (lexer, ')');
+	  lex_force_match (lexer, T_RPAREN);
 	}
       else
         {
@@ -175,11 +175,11 @@ cmd_variable_level (struct lexer *lexer, struct dataset *ds)
 	var_set_measure (v[i], level);
 
 
-      while (lex_token (lexer) == '/')
+      while (lex_token (lexer) == T_SLASH)
 	lex_get (lexer);
       free (v);
 
     }
-  while (lex_token (lexer) != '.');
+  while (lex_token (lexer) != T_ENDCMD);
   return CMD_SUCCESS;
 }
