@@ -10,23 +10,24 @@
 #include <math.h>
 #include <stdbool.h>
 
-#include <data/calendar.h>
-#include <data/case.h>
-#include <data/data-in.h>
-#include <data/data-out.h>
-#include <data/dictionary.h>
-#include <data/procedure.h>
-#include <data/settings.h>
-#include <data/value.h>
-#include <data/variable.h>
-#include <data/vector.h>
-#include <language/expressions/public.h>
-#include <libpspp/compiler.h>
-#include <libpspp/message.h>
-#include <libpspp/misc.h>
-#include <libpspp/str.h>
-#include <math/moments.h>
-#include <math/random.h>
+#include "data/calendar.h"
+#include "data/case.h"
+#include "data/data-in.h"
+#include "data/data-out.h"
+#include "data/dataset.h"
+#include "data/dictionary.h"
+#include "data/settings.h"
+#include "data/value.h"
+#include "data/variable.h"
+#include "data/vector.h"
+#include "language/expressions/public.h"
+#include "libpspp/compiler.h"
+#include "libpspp/i18n.h"
+#include "libpspp/message.h"
+#include "libpspp/misc.h"
+#include "libpspp/str.h"
+#include "math/moments.h"
+#include "math/random.h"
 
 #include "gettext.h"
 #define _(msgid) gettext (msgid)
@@ -48,7 +49,7 @@ static inline double check_errno (double x)
 
 extern const struct substring empty_string;
 
-int compare_string (const struct substring *, const struct substring *);
+int compare_string_3way (const struct substring *, const struct substring *);
 
 double expr_ymd_to_date (double year, double month, double day);
 double expr_ymd_to_ofs (double year, double month, double day);
