@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2007, 2010, 2011  Free Software Foundation
+   Copyright (C) 2007, 2010, 2011, 2012  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,11 +17,11 @@
 #include <config.h>
 
 #include "psppire-dialog.h"
-#include "helper.h"
+#include "builder-wrapper.h"
 #include "psppire-data-window.h"
 #include "psppire-data-editor.h"
 #include "executor.h"
-#include "psppire-var-store.h"
+#include "helper.h"
 #include <ui/syntax-gen.h>
 
 #include "comments-dialog.h"
@@ -103,9 +103,6 @@ comments_dialog (PsppireDataWindow *de)
   GtkWidget *label = get_widget_assert (xml, "column-number-label");
   GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
 
-  PsppireVarStore *vs = NULL;
-
-  g_object_get (de->data_editor, "var-store", &vs, NULL);
 
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (de));
 
@@ -143,7 +140,7 @@ comments_dialog (PsppireDataWindow *de)
   }
 
   cd.xml = xml;
-  g_object_get (vs, "dictionary", &cd.dict, NULL);
+  g_object_get (de->data_editor, "dictionary", &cd.dict, NULL);
 
   g_signal_connect (buffer, "mark-set",
 		    G_CALLBACK (set_column_number), label);
