@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2010, 2011  Free Software Foundation
+   Copyright (C) 2010, 2011, 2012  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,9 +23,9 @@
 #include "psppire-acr.h"
 #include "dialog-common.h"
 
-#include "helper.h"
+#include "builder-wrapper.h"
 #include "executor.h"
-
+#include "helper.h"
 
 #include <gtk/gtk.h>
 
@@ -126,14 +126,11 @@ k_related_dialog (PsppireDataWindow *dw)
   struct k_related_dialog krd;
 
   GtkBuilder *xml = builder_new ("k-related.ui");
-  PsppireVarStore *vs;
 
   GtkWidget *dialog = get_widget_assert   (xml, "k-related-dialog");
 
   GtkWidget *dict_view = get_widget_assert   (xml, "dict-view");
 
-  g_object_get (dw->data_editor, "var-store", &vs, NULL);
-
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (dw));
 
   krd.var_view  = get_widget_assert (xml, "variables-treeview");
@@ -142,7 +139,7 @@ k_related_dialog (PsppireDataWindow *dw)
   krd.kendal =  get_widget_assert (xml, "kendal-checkbutton");
   krd.cochran =  get_widget_assert (xml, "cochran-checkbutton");
 
-  g_object_get (vs, "dictionary", &krd.dict, NULL);
+  g_object_get (dw->data_editor, "dictionary", &krd.dict, NULL);
   g_object_set (dict_view,
 		"model", krd.dict, 
 		"predicate", var_is_numeric,
