@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2006, 2009  Free Software Foundation
+   Copyright (C) 2006, 2009, 2011, 2012  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,28 +24,28 @@
 
 G_BEGIN_DECLS
 
-#define GTK_TYPE_DATA_STORE	       (psppire_data_store_get_type ())
+#define PSPPIRE_TYPE_DATA_STORE	       (psppire_data_store_get_type ())
 
 #define PSPPIRE_DATA_STORE(obj)	\
-                     (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
-								    GTK_TYPE_DATA_STORE, PsppireDataStore))
+  (G_TYPE_CHECK_INSTANCE_CAST ((obj),                                   \
+                               PSPPIRE_TYPE_DATA_STORE, PsppireDataStore))
 
 #define PSPPIRE_DATA_STORE_CLASS(klass) \
-                     (G_TYPE_CHECK_CLASS_CAST ((klass), \
-								 GTK_TYPE_DATA_STORE, \
-                                                                 PsppireDataStoreClass))
+  (G_TYPE_CHECK_CLASS_CAST ((klass),                                    \
+                            PSPPIRE_TYPE_DATA_STORE,                    \
+                            PsppireDataStoreClass))
 
 
 #define PSPPIRE_IS_DATA_STORE(obj) \
-	             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_DATA_STORE))
+  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PSPPIRE_TYPE_DATA_STORE))
 
 #define PSPPIRE_IS_DATA_STORE_CLASS(klass) \
-                     (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_DATA_STORE))
+  (G_TYPE_CHECK_CLASS_TYPE ((klass), PSPPIRE_TYPE_DATA_STORE))
 
 #define PSPPIRE_DATA_STORE_GET_CLASS(obj) \
-                     (G_TYPE_INSTANCE_GET_CLASS ((obj), \
-								   GTK_TYPE_DATA_STORE, \
-								   PsppireDataStoreClass))
+  (G_TYPE_INSTANCE_GET_CLASS ((obj),                                    \
+                              PSPPIRE_TYPE_DATA_STORE,                  \
+                              PsppireDataStoreClass))
 
 typedef struct _PsppireDataStore       PsppireDataStore;
 typedef struct _PsppireDataStoreClass  PsppireDataStoreClass;
@@ -57,7 +57,6 @@ enum dict_signal_handler {
   VARIABLE_INSERTED,
   VARIABLE_CHANGED,
   VARIABLE_DELETED,
-  SIZE_CHANGED,
   n_dict_signals
 };
 
@@ -74,9 +73,6 @@ struct _PsppireDataStore
   PsppireDict *dict;
   struct datasheet *datasheet;
 
-  gboolean show_labels;
-
-  //  gint cf_handler_id [n_cf_signals];
   gint dict_handler_id [n_dict_signals];
 };
 
@@ -96,9 +92,6 @@ void psppire_data_store_set_reader (PsppireDataStore *ds,
 void psppire_data_store_set_dictionary (PsppireDataStore *data_store,
 					PsppireDict *dict);
 
-void psppire_data_store_show_labels (PsppireDataStore *store,
-				     gboolean show_labels);
-
 void psppire_data_store_clear (PsppireDataStore *data_store);
 
 gboolean psppire_data_store_insert_new_case (PsppireDataStore *ds, casenumber posn);
@@ -109,12 +102,17 @@ gboolean psppire_data_store_delete_cases (PsppireDataStore *ds, casenumber first
 
 struct casereader * psppire_data_store_get_reader (PsppireDataStore *ds);
 
-gchar * psppire_data_store_get_string (PsppireDataStore *ds,
-				       casenumber row, glong column);
-
+gchar *psppire_data_store_get_string (PsppireDataStore *,
+                                      glong row, const struct variable *,
+                                      bool use_value_label);
+gboolean psppire_data_store_set_value (PsppireDataStore *,
+                                       casenumber casenum,
+                                       const struct variable *,
+                                       const union value *);
 gboolean psppire_data_store_set_string (PsppireDataStore *ds,
 					const gchar *text,
-					glong row, glong column);
+					glong row, const struct variable *,
+                                        gboolean use_value_label);
 
 
 gboolean psppire_data_store_filtered (PsppireDataStore *ds,
