@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2009, 2010, 2011  Free Software Foundation
+   Copyright (C) 2009, 2010, 2011, 2012  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -68,7 +68,7 @@ psppire_dict_view_finalize (GObject *object)
 {
   PsppireDictView *dict_view = PSPPIRE_DICT_VIEW (object);
 
-  g_object_unref (dict_view->menu);
+  gtk_widget_destroy (dict_view->menu);
 }
 
 /* Properties */
@@ -105,7 +105,10 @@ set_model (PsppireDictView *dict_view)
 {
   GtkTreeModel *model ;
 
-  if ( dict_view->predicate )
+  if ( dict_view->dict == NULL)
+    return;
+
+   if ( dict_view->predicate )
     {
       model = gtk_tree_model_filter_new (GTK_TREE_MODEL (dict_view->dict),
 					 NULL);
@@ -118,9 +121,11 @@ set_model (PsppireDictView *dict_view)
   else
     {
       model = GTK_TREE_MODEL (dict_view->dict);
+      g_object_ref (model);
     }
 
   gtk_tree_view_set_model (GTK_TREE_VIEW (dict_view), model);
+  g_object_unref (model);
 }
 
 static void
@@ -300,12 +305,9 @@ var_description_cell_data_func (GtkTreeViewColumn *col,
   struct variable *var;
   GtkTreeIter iter;
   GtkTreeModel *model;
-  PsppireDict *dict;
 
   dv_get_base_model (top_model, top_iter, &model, &iter);
 
-  dict = PSPPIRE_DICT (model);
-
   gtk_tree_model_get (model,
 		      &iter, DICT_TVM_COL_VAR, &var, -1);
 
@@ -409,10 +411,8 @@ set_tooltip_for_variable (GtkTreeView  *treeview,
   {
     const gchar *tip ;
     GtkTreeModel *m;
-    PsppireDict *dict;
 
     dv_get_base_model (tree_model, NULL, &m, NULL);
-    dict = PSPPIRE_DICT (m);
 
     if ( PSPPIRE_DICT_VIEW (treeview)->prefer_labels )
       tip = var_get_name (var);
