@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2007, 2009, 2010 Free Software Foundation
+   Copyright (C) 2007, 2009, 2010, 2012 Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -67,8 +67,6 @@
 
 #include "psppire-selector.h"
 
-static void psppire_selector_base_finalize (PsppireSelectorClass *, gpointer);
-static void psppire_selector_base_init     (PsppireSelectorClass *class);
 static void psppire_selector_class_init    (PsppireSelectorClass *class);
 static void psppire_selector_init          (PsppireSelector      *selector);
 
@@ -82,6 +80,26 @@ enum  {SELECTED,    /* Emitted when an item is inserted into dest */
 
 static guint signals [n_SIGNALS];
 
+/* Callback for when an item disappears from the source list.
+   By implication, this means that the item has been inserted into the
+   destination.
+ */
+static void
+on_row_deleted (PsppireSelector *selector)
+{
+  g_signal_emit (selector, signals [SELECTED], 0);
+}
+
+/* Callback for when a new item appears in the source list.
+   By implication, this means that an item has been deleted from the
+   destination.
+ */
+static void
+on_row_inserted (PsppireSelector *selector)
+{
+  g_signal_emit (selector, signals [DE_SELECTED], 0);
+}
+
 
 GType
 psppire_selector_get_type (void)
@@ -93,8 +111,8 @@ psppire_selector_get_type (void)
       static const GTypeInfo psppire_selector_info =
       {
 	sizeof (PsppireSelectorClass),
-	(GBaseInitFunc) psppire_selector_base_init,
-        (GBaseFinalizeFunc) psppire_selector_base_finalize,
+	(GBaseInitFunc) NULL, 
+        (GBaseFinalizeFunc) NULL,
 	(GClassInitFunc)psppire_selector_class_init,
 	(GClassFinalizeFunc) NULL,
 	NULL,
@@ -113,18 +131,43 @@ psppire_selector_get_type (void)
 
 static GObjectClass * parent_class = NULL;
 
+
+
+#define SELECTOR_DEBUGGING 0
+
 static void
-psppire_selector_finalize (GObject *obj)
+dump_hash_entry (gpointer key, gpointer value, gpointer obj)
 {
-   /* Chain up to the parent class */
-   G_OBJECT_CLASS (parent_class)->finalize (obj);
+  GList *item = NULL;
+  g_print ("Source %p; ", key);
+
+  for (item = g_list_first (value);
+       item != NULL;
+       item = g_list_next (item))
+    {
+      g_print ("%p(%p) ", item->data, item);
+    }
+  g_print ("\n");
 }
 
+/* This function is for debugging only */
+void 
+psppire_selector_show_map (PsppireSelector *obj)
+{
+  PsppireSelectorClass *class = g_type_class_peek (PSPPIRE_SELECTOR_TYPE);
+
+  g_print ("%s %p\n", __FUNCTION__, obj);
+  g_hash_table_foreach (class->source_hash, dump_hash_entry, obj);
+}
+
+
 
 static void
 psppire_selector_dispose (GObject *obj)
 {
   PsppireSelector *sel = PSPPIRE_SELECTOR (obj);
+  PsppireSelectorClass *class = g_type_class_peek (PSPPIRE_SELECTOR_TYPE);
+  GList *list;
 
   if (sel->dispose_has_run)
     return;
@@ -132,6 +175,21 @@ psppire_selector_dispose (GObject *obj)
   /* Make sure dispose does not run twice. */
   sel->dispose_has_run = TRUE;
 
+  /* Remove ourself from the source map. If we are the only entry for this source
+     widget, that will result in an empty list for the source.  In that case, we
+     remove the entire entry too. */
+  if ((list = g_hash_table_lookup (class->source_hash, sel->source)))
+    {
+      GList *newlist = g_list_remove_link (list, sel->source_litem);
+      g_list_free (sel->source_litem);
+      if (newlist == NULL)
+	g_hash_table_remove (class->source_hash, sel->source);
+      else
+	g_hash_table_replace (class->source_hash, sel->source, newlist);
+
+      sel->source_litem = NULL;
+    }
+  
   g_object_unref (sel->dest);
   g_object_unref (sel->source);
 
@@ -152,6 +210,8 @@ enum
 
 
 static void on_click (GtkButton *b);
+static void on_realize (GtkWidget *selector);
+
 
 static void update_subjects (PsppireSelector *selector);
 
@@ -219,6 +279,7 @@ psppire_selector_class_init (PsppireSelectorClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
   GtkButtonClass *button_class = GTK_BUTTON_CLASS (class);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
   GParamSpec *orientation_spec =
     g_param_spec_enum ("orientation",
 		       "Orientation",
@@ -252,6 +313,8 @@ psppire_selector_class_init (PsppireSelectorClass *class)
 
 
   button_class->clicked = on_click;
+  widget_class->realize = on_realize;
+
 
   object_class->set_property = psppire_selector_set_property;
   object_class->get_property = psppire_selector_get_property;
@@ -294,31 +357,13 @@ psppire_selector_class_init (PsppireSelectorClass *class)
 		  G_TYPE_NONE,
 		  0);
 
-  class->default_selection_funcs = g_hash_table_new (g_direct_hash, g_direct_equal);
-}
-
-
-static void
-psppire_selector_base_init (PsppireSelectorClass *class)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (class);
-
-  object_class->finalize = psppire_selector_finalize;
   object_class->dispose = psppire_selector_dispose;
 
   class->source_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+  class->default_selection_funcs = g_hash_table_new (g_direct_hash, g_direct_equal);
 }
 
 
-
-static void
-psppire_selector_base_finalize(PsppireSelectorClass *class,
-				gpointer class_data)
-{
-  g_hash_table_destroy (class->source_hash);
-  g_hash_table_destroy (class->default_selection_funcs);
-}
-
 /* Callback for when the source treeview is activated (double clicked) */
 static void
 on_row_activate (GtkTreeView       *tree_view,
@@ -352,13 +397,17 @@ on_source_select (GtkTreeSelection *treeselection, gpointer data)
 
 
 static void
-on_realize (PsppireSelector *selector)
+on_realize (GtkWidget *w)
 {
+  PsppireSelector *selector = PSPPIRE_SELECTOR (w);
   PsppireSelectorClass *class = g_type_class_peek (PSPPIRE_SELECTOR_TYPE);
   GtkTreeSelection* selection ;
 
   GList *list = g_hash_table_lookup (class->source_hash, selector->source);
 
+  if (GTK_WIDGET_CLASS (parent_class)->realize)
+    GTK_WIDGET_CLASS (parent_class)->realize (w);
+
   if ( NULL == list)
     return;
 
@@ -391,7 +440,6 @@ psppire_selector_init (PsppireSelector *selector)
   selector->filter = NULL;
 
   selector->arrow = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_NONE);
-  selector->filtered_source = NULL;
 
 
   gtk_container_add (GTK_CONTAINER (selector), selector->arrow);
@@ -408,9 +456,7 @@ psppire_selector_init (PsppireSelector *selector)
   selector->row_activate_id = 0;
   selector->source_select_id  = 0;
 
-  g_signal_connect (selector, "realize",
-		    G_CALLBACK (on_realize), NULL);
-
+  selector->source_litem = NULL;
 }
 
 
@@ -545,6 +591,18 @@ de_select_selection_tree_view (PsppireSelector *selector)
 }
 
 
+/* Callback which causes the filter to be refiltered.
+   Called when the DEST GtkEntry is activated (Enter is pressed), or when it
+   looses focus.
+*/
+static gboolean
+refilter (PsppireSelector *selector)
+{
+  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (selector->source));
+  gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (model));
+  return FALSE;
+}
+
 /* Removes something from the DEST widget */
 static void
 de_select_selection (PsppireSelector *selector)
@@ -562,7 +620,7 @@ de_select_selection (PsppireSelector *selector)
 
   selector->selecting = FALSE;
 
-  gtk_tree_model_filter_refilter (selector->filtered_source);
+  refilter (selector);
 
   g_signal_emit (selector, signals [DE_SELECTED], 0);
 }
@@ -576,14 +634,18 @@ select_selection (PsppireSelector *selector)
   GtkTreeSelection* selection =
     gtk_tree_view_get_selection ( GTK_TREE_VIEW (selector->source));
 
-  GList *selected_rows =
-    gtk_tree_selection_get_selected_rows (selection, NULL);
+  GList *selected_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
 
-  GtkTreeModel *childmodel  = gtk_tree_model_filter_get_model
-    (selector->filtered_source);
+  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (selector->source));
+
+  GtkTreeModel *childmodel = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
 
   g_return_if_fail (selector->select_items);
 
+  if (selector->allow_selection && 
+      ! selector->allow_selection (selector->source, selector->dest))
+    return;
+
   selector->selecting = TRUE;
 
   for (item = g_list_first (selected_rows);
@@ -594,14 +656,12 @@ select_selection (PsppireSelector *selector)
       GtkTreeIter iter;
       GtkTreePath *path  = item->data;
 
-      gtk_tree_model_get_iter (GTK_TREE_MODEL (selector->filtered_source),
-			       &iter, path);
+      g_return_if_fail (model);
 
-      gtk_tree_model_filter_convert_iter_to_child_iter
-	(selector->filtered_source,
-	 &child_iter,
-	 &iter);
+      gtk_tree_model_get_iter (model, &iter, path);
 
+      gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model),
+							&child_iter, &iter);
       selector->select_items (child_iter,
 			      selector->dest,
 			      childmodel,
@@ -612,7 +672,7 @@ select_selection (PsppireSelector *selector)
   g_list_foreach (selected_rows, (GFunc) gtk_tree_path_free, NULL);
   g_list_free (selected_rows);
 
-  gtk_tree_model_filter_refilter (selector->filtered_source);
+  refilter (selector);
 
   g_signal_emit (selector, signals [SELECTED], 0);
 
@@ -642,7 +702,6 @@ on_click (GtkButton *b)
 
   if (GTK_BUTTON_CLASS (parent_class)->clicked)
     GTK_BUTTON_CLASS (parent_class)->clicked (b);
-
 }
 
 static gboolean
@@ -709,50 +768,90 @@ is_source_item_visible (GtkTreeModel *childmodel,
 
 /* set the source widget to SOURCE */
 static void
-set_tree_view_source (PsppireSelector *selector,
-		      GtkTreeView *source)
+set_tree_view_source (PsppireSelector *selector)
 {
   GList *list = NULL;
 
   PsppireSelectorClass *class = g_type_class_peek (PSPPIRE_SELECTOR_TYPE);
   
-  GtkTreeModel *model = gtk_tree_view_get_model (source);
-
-  if ( ! (list = g_hash_table_lookup (class->source_hash, source)))
+  if ( ! (list = g_hash_table_lookup (class->source_hash, selector->source)))
     {
-      selector->filtered_source =
-	GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (model, NULL));
-
-      gtk_tree_view_set_model (source,
-			       GTK_TREE_MODEL (selector->filtered_source));
+      /* Base case:  This widget is currently not the source of 
+	 any selector.  Create a hash entry and make this selector
+	 the first selector in the list */
 
       list = g_list_append (list, selector);
-      g_hash_table_insert (class->source_hash, source, list);
+      g_hash_table_insert (class->source_hash, selector->source, list);
 
-
-      gtk_tree_model_filter_set_visible_func (selector->filtered_source,
-					      is_source_item_visible,
-					      selector,
-					      NULL);
+      /* Save the list item so that it can be removed later */
+      selector->source_litem = list;
     }
   else
     {  /* Append this selector to the list and push the <source,list>
 	  pair onto the hash table */
 
-      selector->filtered_source = GTK_TREE_MODEL_FILTER (model);
-
       if ( NULL == g_list_find (list, selector) )
 	{
 	  if ( selector->primary_requested )
-	    list = g_list_prepend (list, selector);
+	    {
+	      list = g_list_prepend (list, selector);
+	      selector->source_litem = list;
+	    }
 	  else
-	    list = g_list_append (list, selector);
-	  g_hash_table_replace (class->source_hash, source, list);
+	    {
+	      list = g_list_append (list, selector);
+	      selector->source_litem = g_list_last (list);
+	    }
+	  g_hash_table_replace (class->source_hash, selector->source, list);
 	}
     }
 }
 
 
+
+/* This function is a callback which occurs when the
+   SOURCE's model has changed */
+static void
+update_model (
+              GtkTreeView *source,
+              GParamSpec *psp,
+              PsppireSelector *selector
+              )
+{
+  GtkTreeModel *model = gtk_tree_view_get_model (source);
+
+  g_assert (source == GTK_TREE_VIEW (selector->source));
+
+  if (model && (model == g_object_get_data (G_OBJECT (source), "model-copy")))
+    return;
+
+  if (model != NULL) 
+    {      
+      GtkTreeModel *new_model = gtk_tree_model_filter_new (model, NULL); 
+
+      g_object_set_data (G_OBJECT (source), "model-copy", new_model);  
+
+      gtk_tree_view_set_model (source, new_model);
+  
+      gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (new_model),
+                                              is_source_item_visible,
+                                              selector,
+                                              NULL);
+
+      g_signal_connect_swapped (new_model,
+				"row-deleted",
+				G_CALLBACK (on_row_deleted), selector);
+
+      g_signal_connect_swapped (new_model,
+				"row-inserted",
+				G_CALLBACK (on_row_inserted), selector);
+
+      g_object_unref (new_model);
+    }
+}
+
+
+
 /*
    Callback for when the destination treeview's data changes
  */
@@ -766,7 +865,7 @@ on_dest_data_change (GtkTreeModel *tree_model,
 
   if ( selector->selecting) return;
 
-  gtk_tree_model_filter_refilter (selector->filtered_source);
+  refilter (selector);
 }
 
 
@@ -779,7 +878,7 @@ on_dest_data_delete (GtkTreeModel *tree_model,
 
   if ( selector->selecting ) return;
 
-  gtk_tree_model_filter_refilter (selector->filtered_source);
+  refilter (selector);
 }
 
 
@@ -788,11 +887,14 @@ on_dest_model_changed (PsppireSelector *selector)
 {
   GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (selector->dest));
 
-  g_signal_connect (model, "row-changed", G_CALLBACK (on_dest_data_change),
-		    selector);
-
-  g_signal_connect (model, "row-deleted", G_CALLBACK (on_dest_data_delete),
-		    selector);
+  if ( model ) 
+    {
+      g_signal_connect (model, "row-changed", G_CALLBACK (on_dest_data_change),
+			selector);
+      
+      g_signal_connect (model, "row-deleted", G_CALLBACK (on_dest_data_delete),
+			selector);
+    }
 }
 
 /* Set the destination widget to DEST */
@@ -814,16 +916,6 @@ set_tree_view_dest (PsppireSelector *selector,
 }
 
 
-/* Callback which causes the filter to be refiltered.
-   Called when the DEST GtkEntry is activated (Enter is pressed), or when it
-   looses focus.
-*/
-static gboolean
-refilter (PsppireSelector *selector)
-{
-  gtk_tree_model_filter_refilter (selector->filtered_source);
-  return FALSE;
-}
 
 
 /* Callback for when the DEST GtkEntry is selected (clicked) */
@@ -839,27 +931,6 @@ on_entry_dest_select (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 }
 
 
-/* Callback for when an item disappears from the source list.
-   By implication, this means that the item has been inserted into the
-   destination.
- */
-static void
-on_row_deleted (PsppireSelector *selector)
-{
-  g_signal_emit (selector, signals [SELECTED], 0);
-}
-
-/* Callback for when a new item appears in the source list.
-   By implication, this means that an item has been deleted from the
-   destination.
- */
-static void
-on_row_inserted (PsppireSelector *selector)
-{
-  g_signal_emit (selector, signals [DE_SELECTED], 0);
-}
-
-
 
 /* Set DEST to be the destination GtkEntry widget */
 static void
@@ -879,11 +950,6 @@ set_entry_dest (PsppireSelector *selector,
 		    selector);
 
 
-  g_signal_connect_swapped (selector->filtered_source, "row-deleted",
-		    G_CALLBACK (on_row_deleted), selector);
-
-  g_signal_connect_swapped (selector->filtered_source, "row-inserted",
-		    G_CALLBACK (on_row_inserted), selector);
 }
 
 static void
@@ -896,11 +962,10 @@ set_default_filter (PsppireSelector *selector)
     }
 }
 
+
 static void
 update_subjects (PsppireSelector *selector)
 {
-  GtkTreeModel *model = NULL;
-
   if ( NULL == selector->dest )
     return;
 
@@ -909,17 +974,15 @@ update_subjects (PsppireSelector *selector)
   if ( NULL == selector->source )
     return;
 
-  g_signal_connect_swapped (selector->source, "notify::model",
-			    G_CALLBACK (update_subjects), selector);
-
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (selector->source));
-
-  if ( NULL == model)
-    return;
-
-
   if ( GTK_IS_TREE_VIEW (selector->source))
-    set_tree_view_source (selector, GTK_TREE_VIEW (selector->source) );
+    {
+      set_tree_view_source (selector);
+
+      g_signal_connect (selector->source, "notify::model", 
+                              G_CALLBACK (update_model), selector); 
+
+      update_model (GTK_TREE_VIEW (selector->source), 0, selector);
+    }
   else
     g_error ("Unsupported source widget: %s", G_OBJECT_TYPE_NAME (selector->source));
 
