@@ -44,9 +44,15 @@ struct _PsppireVarView
 {
   GtkTreeView parent;
 
-  GtkListStore *list;
+  /* Private */
+  GtkListStore **list;
   
+  GType *cols;
   gint *nums;
+  gint n_cols;
+
+  gint n_lists;
+  gint l_idx;
 };
 
 struct _PsppireVarViewClass
@@ -58,6 +64,8 @@ struct _PsppireVarViewClass
 GType      psppire_var_view_get_type        (void);
 
 gint psppire_var_view_append_names (PsppireVarView *vv, gint column, GString *string);
+struct string;
+gint psppire_var_view_append_names_str (PsppireVarView *vv, gint column, struct string *);
 
 gboolean psppire_var_view_get_iter_first (PsppireVarView *vv, GtkTreeIter *iter);
 
@@ -65,6 +73,14 @@ gboolean psppire_var_view_get_iter_next (PsppireVarView *vv, GtkTreeIter *iter);
 
 const struct variable * psppire_var_view_get_variable (PsppireVarView *vv, gint column, GtkTreeIter *iter);
 
+void     psppire_var_view_push_model (PsppireVarView *vv);
+
+gboolean psppire_var_view_set_current_model (PsppireVarView *vv, gint n);
+
+void psppire_var_view_clear (PsppireVarView *vv);
+
+GtkWidget* psppire_var_view_new (void);
+
 
 
 G_END_DECLS
