@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2008, 2009, 2010, 2011  Free Software Foundation
+   Copyright (C) 2008, 2009, 2010, 2011, 2013  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -73,6 +73,7 @@ struct _PsppireWindow
   guint insert_handler;
   guint remove_handler;
 
+  gboolean added_separator;
   gboolean dirty;
   GTimeVal savetime;
 };
@@ -90,7 +91,7 @@ struct _PsppireWindowIface
 
   void (*save) (PsppireWindow *w);
   void (*pick_filename) (PsppireWindow *);
-  gboolean (*load) (PsppireWindow *w, const gchar *);
+  gboolean (*load) (PsppireWindow *w, const gchar *, gpointer hint);
 };
 
 
@@ -111,7 +112,7 @@ gint psppire_window_query_save (PsppireWindow *);
 
 void psppire_window_save (PsppireWindow *w);
 void psppire_window_save_as (PsppireWindow *w);
-gboolean psppire_window_load (PsppireWindow *w, const gchar *file);
+gboolean psppire_window_load (PsppireWindow *w, const gchar *file, gpointer hint);
 void psppire_window_open (PsppireWindow *de);
 GtkWidget *psppire_window_file_chooser_dialog (PsppireWindow *toplevel);
 
