@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2007, 2010, 2011  Free Software Foundation
+   Copyright (C) 2007, 2010, 2011, 2012  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "executor.h"
 #include "psppire-data-window.h"
 #include "dict-display.h"
+#include "builder-wrapper.h"
 #include "helper.h"
 
 #include "dialog-common.h"
@@ -35,10 +36,6 @@
 #define N_(msgid) msgid
 
 
-/* FIXME: These shouldn't be here */
-#include "psppire-var-store.h"
-
-
 static gchar * generate_syntax (PsppireDict *dict, GtkBuilder *xml);
 
 static void
@@ -83,15 +80,11 @@ transpose_dialog (PsppireDataWindow *de)
 
   GtkBuilder *xml = builder_new ("psppire.ui");
 
-  PsppireVarStore *vs = NULL;
-
   GtkWidget *dialog = get_widget_assert (xml, "transpose-dialog");
   GtkWidget *source = get_widget_assert (xml, "source-treeview");
   GtkWidget *selector2 = get_widget_assert (xml, "psppire-selector3");
 
-  g_object_get (de->data_editor, "var-store", &vs, NULL);
-
-  g_object_get (vs, "dictionary", &dict, NULL);
+  g_object_get (de->data_editor, "dictionary", &dict, NULL);
   g_object_set (source, "model", dict, NULL);
 
   psppire_selector_set_filter_func (PSPPIRE_SELECTOR (selector2),
