@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2007, 2010, 2011  Free Software Foundation
+   Copyright (C) 2007, 2010, 2011, 2012  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include "executor.h"
 #include "psppire-data-window.h"
 #include "dict-display.h"
+#include "builder-wrapper.h"
 #include "helper.h"
 
 #include <gtk/gtk.h>
@@ -31,8 +32,6 @@
 #define N_(msgid) msgid
 
 
-#include "psppire-var-store.h"
-
 struct weight_cases_dialog
 {
   PsppireDict *dict;
@@ -118,10 +117,7 @@ weight_cases_dialog (PsppireDataWindow *de)
 
   GtkWidget *selector = get_widget_assert (xml, "weight-cases-selector");
 
-  PsppireVarStore *vs = NULL;
-
-  g_object_get (de->data_editor, "var-store", &vs,  NULL);
-  g_object_get (vs, "dictionary", &wcd.dict, NULL);
+  g_object_get (de->data_editor, "dictionary", &wcd.dict, NULL);
 
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (de));
 
