@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 1997-2000, 2006-2007, 2009-2012 Free Software Foundation, Inc.
+   Copyright (C) 1997-2000, 2006-2007, 2009-2013 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -96,7 +96,7 @@ main (int argc, char **argv)
   random_init ();
 
   lexer = lex_create ();
-  the_session = session_create ();
+  the_session = session_create (NULL);
   dataset_create (the_session, "");
 
   parser = argv_parser_create ();
