@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 2007, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2009, 2010 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,9 +40,6 @@
 #define UNUSED
 #endif
 
-/* Currently running test. */
-static const char *test_name;
-
 /* Exit with a failure code.
    (Place a breakpoint on this function while debugging.) */
 static void
@@ -58,8 +55,7 @@ check_func (bool ok, int line)
 {
   if (!ok)
     {
-      printf ("%s:%d: Check failed in %s test\n",
-              __FILE__, line, test_name);
+      fprintf (stderr, "%s:%d: check failed\n", __FILE__, line);
       check_die ();
     }
 }
@@ -288,6 +284,7 @@ test_insert_delete_strides (void)
   deletions = xnmalloc (cnt, sizeof *deletions);
   for (stride = strides; stride < strides + stride_cnt; stride++)
     {
+      printf ("%lu\n", *stride);
       for (offset = offsets; offset < offsets + offset_cnt; offset++)
         {
           int k;
@@ -306,8 +303,6 @@ test_insert_delete_strides (void)
           test_insert_delete (insertions, insertions, cnt);
           test_insert_delete (insertions, deletions, cnt);
         }
-      putchar ('.');
-      fflush (stdout);
     }
   free (insertions);
   free (deletions);
@@ -428,27 +423,62 @@ test_random_insert_delete (void)
   sparse_array_destroy (spar);
   free (has_values);
 }
-
+
 /* Main program. */
 
-/* Runs TEST_FUNCTION and prints a message about NAME. */
-static void
-run_test (void (*test_function) (void), const char *name)
-{
-  test_name = name;
-  putchar ('.');
-  fflush (stdout);
-  test_function ();
-}
+struct test
+  {
+    const char *name;
+    const char *description;
+    void (*function) (void);
+  };
+
+static const struct test tests[] =
+  {
+    {
+      "random-insert-delete",
+      "random insertions and deletions",
+      test_random_insert_delete
+    },
+    {
+      "insert-delete-strides",
+      "insert in ascending order with strides and offset",
+      test_insert_delete_strides
+    },
+  };
+
+enum { N_TESTS = sizeof tests / sizeof *tests };
 
 int
-main (void)
+main (int argc, char *argv[])
 {
-  run_test (test_random_insert_delete,
-            "random insertions and deletions");
-  run_test (test_insert_delete_strides,
-            "insert in ascending order with strides and offset");
-  putchar ('\n');
+  int i;
 
-  return 0;
+  if (argc != 2)
+    {
+      fprintf (stderr, "exactly one argument required; use --help for help\n");
+      return EXIT_FAILURE;
+    }
+  else if (!strcmp (argv[1], "--help"))
+    {
+      printf ("%s: test sparse array library\n"
+              "usage: %s TEST-NAME\n"
+              "where TEST-NAME is one of the following:\n",
+              argv[0], argv[0]);
+      for (i = 0; i < N_TESTS; i++)
+        printf ("  %s\n    %s\n", tests[i].name, tests[i].description);
+      return 0;
+    }
+  else
+    {
+      for (i = 0; i < N_TESTS; i++)
+        if (!strcmp (argv[1], tests[i].name))
+          {
+            tests[i].function ();
+            return 0;
+          }
+
+      fprintf (stderr, "unknown test %s; use --help for help\n", argv[1]);
+      return EXIT_FAILURE;
+    }
 }
